/*---------------------------------------------------------------------------*/
/* a s s e s s   --   assess the output of an edge detector                  */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Standard include files                                                    */
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <pbmplus.h>


/*---------------------------------------------------------------------------*/
/* Local data types                                                          */
/*---------------------------------------------------------------------------*/

typedef enum {FALSE, TRUE} logical;
typedef short int pixel_t;


/*---------------------------------------------------------------------------*/
/* Global variables                                                          */
/*---------------------------------------------------------------------------*/

char *progname = "<unknown>";


/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

pixel_t **read_im (FILE *, int *, int *);


/*---------------------------------------------------------------------------*/
/* M a i n   p r o g r a m                                                   */
/*---------------------------------------------------------------------------*/

int
main (int argc, char *argv[])
{
  FILE *fin;
  int nback = 0, nfore = 0;
  int m, n, i, j;
  pixel_t **im, v;


  /* Handle the command-line argument, if any. */

  progname = argv[0];
  if (argc == 2) {
    fin = fopen (argv[1], "rb");
    if (fin == (FILE *)NULL) {
      fprintf (stderr, "Cannot open \"%s\".\n", argv[1]);
      exit (EXIT_FAILURE);
    }
  } else {
    fin = stdin;
  }

  /* Read in the image and process it. */

  im = read_im (fin, &m, &n);
  for (j = 0; j < n; j++) {
    for (i = 0; i < m; i++) {
      v = im[j][i];
      if (v < 10) {
	nback++;
      } else {
	nfore++;
      }
    }
  }
  printf ("%d foreground %d background\n", nfore, nback);
  return EXIT_SUCCESS;
}


/*---------------------------------------------------------------------------*/
/* c r e a t e _ i m   --   create an image structure                        */
/*---------------------------------------------------------------------------*/

pixel_t **
create_im (int m, int n)
{
  int i;
  pixel_t **p;

  p = (pixel_t **)malloc(n * sizeof(pixel_t *));
  if (p == (pixel_t **)NULL)
    return p;

  for (i = 0; i < n; i++) {
    p[i] = (pixel_t *)malloc(m * sizeof(pixel_t));
    if( p[i] == (pixel_t *)NULL)
      return (pixel_t **)0;
  }
  return p;
}


/*---------------------------------------------------------------------------*/
/* r e a d _ i m   --   read in an image from a PGM file                     */
/*---------------------------------------------------------------------------*/

pixel_t **
read_im (FILE *f, int *m, int *n)
{
  int i, j, format;
  gray maxval, *buf;
  pixel_t **p;

  pgm_readpgminit (f, m, n, &maxval, &format);
  p = create_im (*m, *n);

  buf = pgm_allocrow (*m);
  for (j = 0; j < *n; j++) {
    pgm_readpgmrow (f, buf, *m, maxval, format);
    for (i = 0; i < *m; i++) {
      p[j][i] = buf[i];
    }
  }
  pgm_freerow (buf);
  return p;
}
